import argparse
import os
from .utils import converter
from .templates import Templates

def generate_service(filepath):
    if not os.path.exists(filepath):
        os.makedirs(filepath)

    basename = converter.to_snake_case(os.path.basename(filepath))

    service_filename = os.path.join(filepath, f"{basename}_service.py")

    if os.path.exists(service_filename):
        index = 1
        while True:
            unique_filename = f"_service_{index}.py"
            service_filename = os.path.join(filepath, f"{basename}{unique_filename}")
            
            if not os.path.exists(service_filename):
                break

            index += 1

    with open(service_filename, 'w') as file:
        file.write(Templates(filepath).service_template())

        print(f"Service generated at {service_filename}")

def main():
    parser = argparse.ArgumentParser(description="Generate a wavebox framework file")
    parser.add_argument('spawn', type=bool, help='Pass this argument to generate file')
    parser.add_argument('service', type=bool, help="Pass this argument to create service file")
    parser.add_argument('filepath', type=str, help='Directory where the service file will be generated')

    args = parser.parse_args()
    if args.spawn is True:
        if args.service is True:
            generate_service(args.filepath)
    


import os
from ..utils import converter


class Templates:
    def __init__(self, filepath):
        self._filepath = filepath
        
    def service_template(self):
        return f"""\
#This is a generated service template
from flask import Blueprint
from pydantic import BaseModel

class {converter.to_camel_case_with_capitalize(os.path.basename(self._filepath))}Service(Services):
    
    def _validate(args):
        pass
    
    def _logics(self, req) -> (BaseModel, int):
        pass

    def retrieve(self, req: BaseModel) -> (BaseModel, int):
        pass
        
class {converter.to_camel_case_with_capitalize(os.path.basename(self._filepath))}Controller(Controllers, {converter.to_camel_case_with_capitalize(os.path.basename(self._filepath))}Service):

    def __init__(self, blueprint: Blueprint, endpoint: str, methods: List[str]):
        super().__init__(blueprint, endpoint, methods)

    def controller(self):
        try:
            return super().done()
        except FundamentalException as e:
            return super().catcher(err_response, e)
"""


import re


def to_camel_case_with_capitalize(text):
    # Split the text into words using spaces or underscores as separators
    words = re.split(r'[-_\s]+', text)

    # Capitalize the first letter of each word
    camel_case_words = [word.capitalize() for word in words]

    # Join the words together to form the camel case string
    camel_case_text = ''.join(camel_case_words)

    return camel_case_text

def to_snake_case(text):
   # Replace hyphens and spaces with underscores
    text = re.sub(r'[-\s]', '_', text)
    
    # Split the text into words based on uppercase letters and underscores
    words = re.split(r'([A-Z]+|[a-z]+|[_]+)', text)
    
    # Filter out empty strings and convert to lowercase
    snake_case_words = [word.lower() for word in words if word and word not in ["-", "_"]]

    # Join the words together with underscores to form the snake case string
    snake_case_text = '_'.join(snake_case_words)

    return snake_case_text


from setuptools import setup, find_packages

setup(
    name='waveboxgen',
    version='0.1-dev',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'waveboxgen = waveboxgen.main:main'
        ]
    }
)